/*========================================================================================================*
* PINEWOOD DERBY CAR SCALE w/ CG                                                 Version 1 - 21 July 2021 *
* Created by: Bill van Leeuwen (email: Billv923@outlook.com) for the Arduino UNO microcontroller          *
*                                                                                                         *
* This sketch uses two HX711 load cells to measure the weight at the front and rear axle of a pinewood    *
* derby car. This will allow calculation of the car's CG (center-of-gravity) to determine if it is within *
* its optimum range for best performance. Weights and CG are displayed on a 4-line LCD display. Two pins  *
* are required for the HX711 serial interface. Load Cell 1 (Scale 1) uses pins D2 (Dout) & D3 (SCK).      *
* Load Cell 2 (Scale 2) uses pins D4 (Dout) & D4 (SCK). The LCD I2C pins used on the UNO are assigned by  *
* the LiquidCrystal_I2C library and are as follows: SDA=Pin A4, SCL=Pin A5.                               *
*========================================================================================================*/

//Include the following libraries
#include <Wire.h>                                   //Required for the LCD display
#include <LiquidCrystal_I2C.h>                      //Required for the LCD display
#include <HX711_ADC.h>                              //Required for the two HX711 load cells
#include <EEPROM.h>                                 //Used to store calibration values in the Arduino EEPROM

//LiquidCrystal_I2C lcd(0x27,20,4);                 //Set the LCD address to 0x27 for a 20 chars and 4 line display
LiquidCrystal_I2C lcd(0x3F,20,4);                   //Set the LCD address to 0x3F for a 20 chars and 4 line display

#define Upbtn      8                                //Up pushbutton pin D8
#define Dnbtn      9                                //Down pushbutton pin D9
#define Enterbtn  10                                //Reset pushbutton pin D10
#define Cal_Sw1   11                                //Load cell 1 calibration mode switch pin D11
#define Cal_Sw2   12                                //Load cell 2 calibration mode switch pin D12

//HX711 constructor (dout pin, sck pin)
HX711_ADC Scale1(2,3);                              //HX711 load cell 1 (Rear axle)
HX711_ADC Scale2(4,5);                              //HX711 load cell 2 (Front axle)

boolean debug = false;                              //Set to true if debugging
boolean doTare = true;                              //Set to true to do tare function at startup
boolean cal1Flag;                                   //True if Scale1 is calibrated
boolean cal2Flag;                                   //True if Scale2 is calibrated
boolean useEepromVals = true;                       //True if using calibration values stored in EEPROM
boolean eepromNeedsUpdate = false;                  //True if EEPROM validity value needs to be updated
byte Scale1_rdy = 0;                                //Scale1 ready flag
byte Scale2_rdy = 0;                                //Scale2 ready flag
byte eepromTest_value = 0xA5;                       //EEPROM validity test value
int i;                                              //Working integer variable
int eepromAddress1 = 0;                             //EEPROM Address for storing Scale1 calibration factor
int eepromAddress2 = 0x04;                          //EEPROM Address for storing Scale2 calibration factor
int eepromAddress3 = 0x08;                          //EEPROM Address for storing calibration weight
int eepromAddress4 = 0x12;                          //EEPROM Address for storing EEPROM validity test value
int Disp_Refresh;                                   //Display refresh rate in milliseconds
int waitCount = 0;                                  //Working variable
long Clk_Time;                                      //Used to time Up & Down button functions
long Disp_Time = 0;                                 //Used to determine when to display scale results
unsigned long stabilizingtime = 2000;               //Improve tare precision by adding stabilizing time
float calFactor1;                                   //calibration factor for Scale1
float calFactor2;                                   //calibration factor for Scale2
float newValue;                                     //Temporary floating point variable
float calMass;                                      //Weight of the object used to calibrate the scale
float Scale1_value;                                 //Measured Scale1 weight value
float Scale2_value;                                 //Measured Scale2 weight value
float Scale_Sum;                                    //Sum of Scale1 & Scale2 weights
float wheelBase;                                    //Car axle separation in inches
float CG;                                           //Car Center of Gravity in inches as measured from rear axle
float Ocg;                                          //Optimum CG (= 0.171 * wheel base
String serial_data;                                 //Incoming serial data variable

//================================================== SETUP ROUTINE ==================================================
void setup() {
  calFactor1 = 25228.3;                             //Default Scale1 Calibration value
  calFactor2 = 23571.7;                             //Default Scale2 Calibration value
  calMass = 2.72;                                   //Preset calibration mass in ounces
  wheelBase = 4.375;                                //Preset axle separation in inches (default = 4.375)
  pinMode(Upbtn, INPUT_PULLUP);                     //Setup Up button
  pinMode(Dnbtn, INPUT_PULLUP);                     //Setup Down button
  pinMode(Enterbtn, INPUT_PULLUP);                  //Setup Enter button
  pinMode(Cal_Sw1, INPUT_PULLUP);                   //Setup Load cell 1 (Scale1) calibration mode switch
  pinMode(Cal_Sw2, INPUT_PULLUP);                   //Setup Load cell 2 (Scale2) calibration mode switch
  Serial.begin(115200);                             //Start serial interface at 115200 baud
  Scale1.begin();                                   //Start Scale1
  Scale2.begin();                                   //Start Scale2
  while ((Scale1_rdy + Scale2_rdy) < 2) {           //Run startup, stabilization and tare, both modules simultaneously
    if (!Scale1_rdy) Scale1_rdy = Scale1.startMultiple(stabilizingtime, doTare);
    if (!Scale2_rdy) Scale2_rdy = Scale2.startMultiple(stabilizingtime, doTare);
  }
  if (useEepromVals) {                              //Do if useEepromVals flag is true
    if (EEPROM.read(eepromAddress4) == eepromTest_value) {  //Is EEPROM data valid
      EEPROM.get(eepromAddress1, newValue);         //Get Scale1 calibration factor stored in EEPROM
      if (!isnan(newValue) && newValue>-100000.0 && newValue<100000.0) {  //Check for valid value
        calFactor1 = newValue;                      //Update Scale1 calibration factor from EEPROM
        Scale1.setCalFactor(calFactor1);            //Set Scale1 calibration value (float)
        cal1Flag = true;                            //Set Scale1 calibrated flag to true
      }
      EEPROM.get(eepromAddress2, newValue);         //Get Scale2 calibration factor stored in EEPROM
      if (!isnan(newValue) && newValue>-100000.0 && newValue<100000.0) {  //Check for valid value
        calFactor2 = newValue;                      //Update Scale2 calibration factor from EEPROM
        Scale2.setCalFactor(calFactor2);            //Set Scale2 calibration value (float)
        cal2Flag = true;                            //Set Scale2 calibrated flag to true
      }
      EEPROM.get(eepromAddress3, newValue);         //Get calibration mass stored in EEPROM
      if (!isnan(newValue) && newValue>0.1 && newValue<5.0) { //Check for valid value
        calMass = newValue;                         //Update calibration mass from EEPROM
      }
    }
    else {                                          //EEPROM data not valid
      cal1Flag = false;                             //Set Scale1 as not calibrated
      cal2Flag = false;                             //Set Scale2 as not calibrated
      eepromNeedsUpdate = true;                     //Set to signify EEPROM validity value needs update
      Scale1.setCalFactor(1.0);                     //Set default Scale1 cal value to 1.0 (float)
      Scale2.setCalFactor(1.0);                     //Set default Scale2 cal value to 1.0 (float)
    }
  }
  if (debug) {                                      //Send data to the serial monitor if debug flag is set
    Serial.print("EEPROM Addr: ");
    Serial.print(eepromAddress1);
    Serial.print("  EEPROM Value: ");
    Serial.println(calFactor1);
    Serial.print("EEPROM Addr: ");
    Serial.print(eepromAddress2);
    Serial.print("  EEPROM Value: ");
    Serial.println(calFactor2);
  }

  lcd.init();                                       //Begin LCD display
  lcd.backlight();                                  //Power ON the back light
  //lcd.backlight();                                //Do again to Power OFF the back light
  
  if (debug) Serial.println("Startup + tare is complete");
  Disp_Refresh = 2000;                              //Set Display Refresh rate to 2 seconds
  Disp_Time = millis();                             //Preset the display timer
}

//============================================= MAIN LOOP ===============================================
void loop(void) {
  //NOTE: Scale update needs to be called at least as often as HX711 sample rate; >10Hz@10SPS, >80Hz@80SPS
  //Longer delays in sketch will reduce effective sample rate. Hence, routines that wait for user input
  //contain scale update commands to ensure they are called often enough.
  Scale1.update();
  Scale2.update();

  //Display Weight & CG routine
  if (millis() > Disp_Time) {                       //Do only if 2 seconds has elapsed since last display
    Scale1_value = Scale1.getData();                //Get Scale1 weight reading
    if (Scale1_value < 0.0) Scale1_value = 0.0;     //Don't allow negative values
    Scale2_value = Scale2.getData();                //Get Scale2 weight reading
    if (Scale2_value < 0.0) Scale2_value = 0.0;     //Don't allow negative values
    Scale_Sum = Scale1_value + Scale2_value;        //Calculate total weight
    CG = 0.0;                                       //Preset CG to zero
    if (Scale_Sum>0.01) {                           //Check sum to avoid divide by zero & random CG calcs
      CG = Scale2_value/Scale_Sum * wheelBase;      //CG = Distance forward of the rear axle
    }
    if (cal1Flag && cal2Flag) {                     //Are both scales calibrated?
      Display_Weights();                            //Yes - Write weight & CG data to the display
    }
    else {
      Display_Cal_Warning();                        //No - Display warning message
    }
    Disp_Time = millis() + Disp_Refresh;            //Set next display time
  }

  //Rear Axle (left) Scale Calibration Routine
  if (digitalRead(Cal_Sw1)==0) {                    //Check for Cal1 Switch set
    Calibrate_Scale(1);                             //Call Subroutine to calibrate Scale1
  }

  //Front Axle (right) Scale Calibration Routine 
  if (digitalRead(Cal_Sw2)==0) {                    //Check for Cal2 Switch set
    Calibrate_Scale(2);                             //Call Subroutine to calibrate Scale2
  }

  //Check for Wheel Base Increase Request
  if (digitalRead(Upbtn)==0 && digitalRead(Enterbtn)==1){
    Clk_Time = millis() + 1500;                     //Get CPU clock time + 1.5 seconds
    Display_WheelBase();                            //Call subroutine to display wheel base
    delay(500);                                     //Insert delay to debounce button
    while (digitalRead(Upbtn)==0 && digitalRead(Enterbtn)==1){ //Do while only Up button is being held
      if (millis() > Clk_Time) {                    //Has Up button been held for >0.75 seconds?
        wheelBase += 0.125;                         //Increase wheelBase by 1/8 inch
        if (wheelBase > 7.0) {                      //Limit maximum wheelBase to 7.0 inches
          wheelBase = 7.0;
        }
        Display_WheelBase();                        //Display updated wheel base
        Scale1.update();                            //Do a Scale1 update
        Scale2.update();                            //Do a Scale2 update
        delay(750);                                 //Insert delay before we loop again
      }
    }
  }
  
  //Check for Wheel Base Decrease Request
  if (digitalRead(Dnbtn)==0 && digitalRead(Enterbtn)==1){
    Clk_Time = millis() + 1500;                     //Get CPU clock time + 1.5 seconds
    Display_WheelBase();                            //Call subroutine to display wheel base
    delay(500);                                     //Insert delay to debounce button
    while (digitalRead(Dnbtn)==0 && digitalRead(Enterbtn)==1){ //Do while only Down button is being held
      if (millis() > Clk_Time) {                    //Has Up button been held for >0.75 seconds?
        wheelBase -= 0.125;                         //Decrease wheelBase by 1/8 inch
        if (wheelBase < 2.0) {                      //Limit minimum wheelBase to 2.0 inches
          wheelBase = 2.0;
        }
        Display_WheelBase();                        //Display updated wheel base
        Scale1.update();                            //Do a Scale1 update
        Scale2.update();                            //Do a Scale2 update
        delay(750);                                 //Insert delay before we loop again
      }
    }
  }

  //Check for Calibration Weight Adjust & Reset Request
  while (digitalRead(Enterbtn)==0) {                //Do while Enter button is depressed
    delay(500);                                     //Insert delay to debounce button
    waitCount++;                                    //Count number of 0.5 second wait times
    //Check for Set Calibration Weight Increase Request
    if (digitalRead(Upbtn)==0) {                    //Do if Up button is depressed
      Clk_Time = millis() + 1500;                   //Get CPU clock time + 1.5 seconds
      Display_Cal_Weight();                         //Call subroutine to display calibration weight
      waitCount = 0;                                //Reset wait counter
      delay(500);                                   //Insert delay to debounce button
      while (digitalRead(Upbtn)==0 && digitalRead(Enterbtn)==0){ //Do while Up & Enter buttons are being held
        if (millis() > Clk_Time) {                  //Has Up button been held for >0.5 seconds?
          calMass += 0.01;                          //Increase calibration mass by 0.01 ounces
          if (calMass > 5.0) {                      //Limit maximum calibration mass to 5.0 ounces
            calMass = 5.0;
          }
          Display_Cal_Weight();                     //Display updated calibration weight
          Scale1.update();                          //Do a Scale1 update
          Scale2.update();                          //Do a Scale2 update
          Clk_Time = millis() + 500;                //Update delay time 0.5 seconds before we repeat
        }
      }
      EEPROM.put(eepromAddress3, calMass);          //Update stored calibration mass
    }
    //Check for Set Calibration Weight Decrease Request
    if (digitalRead(Dnbtn)==0) {                    //Do if Down button is depressed
      Clk_Time = millis() + 1500;                   //Get CPU clock time + 1.5 seconds
      Display_Cal_Weight();                         //Call subroutine to display calibration weight
      waitCount = 0;                                //Reset wait counter
      delay(500);                                   //Insert delay to debounce button
      while (digitalRead(Dnbtn)==0 && digitalRead(Enterbtn)==0){ //Do while Down & Enter buttons are being held
        if (millis() > Clk_Time) {                  //Has Down button been held for >0.5 seconds?
          calMass -= 0.01;                          //Decrease calibration mass by 0.01 ounces
          if (calMass <= 0.5) {                     //Limit minimum calibration mass to 0.5 ounces
            calMass = 0.5;
          }
          Display_Cal_Weight();                     //Display updated calibration weight
          Scale1.update();                          //Do a Scale1 update
          Scale2.update();                          //Do a Scale2 update
          Clk_Time = millis() + 500;                //Update delay time 0.5 seconds before we repeat
        }
      }
      EEPROM.put(eepromAddress3, calMass);           //Update stored calibration mass
    }
    //Check for Reset Request
    if (waitCount > 4) {                            //Do if Upbtn or Dnbtn not pressed in allotted time
      lcd.clear();                                  //Clear the screen
      lcd.setCursor(2,1);                           //Set cursor to column 3, row 2
      lcd.print("PERFORMING RESET");                //Display screen title
      waitCount = 0;                                //Reset wait counter
      Scale1_rdy = Scale2_rdy = 0;                  //Clear scale ready flags
      while ((Scale1_rdy + Scale2_rdy) < 2) {       //Run startup, stabilization and tare, both modules simultaniously
        if (!Scale1_rdy) Scale1_rdy = Scale1.startMultiple(stabilizingtime, doTare);
        if (!Scale2_rdy) Scale2_rdy = Scale2.startMultiple(stabilizingtime, doTare);
      }
      Scale1.setCalFactor(calFactor1);              //Set Scale1 calibration value (float)
      Scale2.setCalFactor(calFactor2);              //Set Scale2 calibration value (float)
      wheelBase = 4.375;                            //Reset wheel base to default of 4.375 inches
      Disp_Time = millis() + Disp_Refresh;          //Reset next display time
    }
  }

  if (eepromNeedsUpdate) {                          //Does EEPROM validity value need updating?
    if (cal1Flag && cal2Flag) {                     //Do if both scales have been calibrated
      EEPROM.write(eepromAddress4, eepromTest_value); //Update EEPROM validity value
      eepromNeedsUpdate = false;                    //Set false so not to repeat
    }
  }

  if (debug) {                                      //Do if debug flag is set
    serial_data = get_serial_data();                //Check terminal for input
    if (serial_data.indexOf("dv") >= 0) {           //Check for display values  "dv" request
      Serial.print("Scale1 val: ");
      Serial.print(Scale1_value);
      Serial.print("   Scale2 val: ");
      Serial.print(Scale2_value);
      Serial.print("   Total val: ");
      Serial.println(Scale_Sum);
      Serial.print("CG: ");
      Serial.print(CG);
      Serial.println(" inches");
      Serial.print("Scale1 cal value: ");
      Serial.print(calFactor1);
      Serial.print("   Scale2 cal value: ");
      Serial.println(calFactor2);
    }
    if (serial_data.indexOf("de") >= 0) {           //Check for display EEPROM "de" values request
      EEPROM.get(eepromAddress1, newValue);         //Get Scale1 stored calibration factor from EEPROM
      Serial.print("Scale1 EEPROM cal value: ");    //Display Scale1 calibration factor
      Serial.print(newValue);
      EEPROM.get(eepromAddress2, newValue);         //Get Scale2 stored calibration factor from EEPROM
      Serial.print("   Scale2 EEPROM cal value: "); //Display Scale2 calibration factor
      Serial.println(newValue);
      EEPROM.get(eepromAddress3, newValue);         //Get calibration mass value from EEPROM
      Serial.print("Stored EEPROM calibration mass: "); //Display calibration mass
      Serial.print(newValue);
      Serial.println(" Oz.");
    }
    if (serial_data.indexOf("ee") >= 0) {           //Check for erase EEPROM request
      for (i=0; i<16; i++) {                        //loop thru address 0 - 15
        EEPROM.write(eepromAddress4, 0x00);         //Erase EEPROM values
      }
      Serial.println("EEPROM erased");              //Display Scale1 calibration factor
    }
  }
  delay(100);                                       //Wait 0.1 seconds before we check again
}

//======================================== SUBROUTINES & FUNCTIONS ========================================
//Function: Read Serial Data
String get_serial_data() {
  String Str = "";
  if (Serial.available() > 0) {                     //Check for new serial data
    Str = Serial.readStringUntil('\n');             //Get it
    return Str;                                     //And return to calling routine with it
  }
}

//Subroutine - Wait for ENTER button press/release
void Waitfor_Enter_Button() {
  while (digitalRead(Enterbtn)==1){                 //Check ENTER button status
    delay(50);                                      //Wait for ENTER button press
    Scale1.update();                                //Do a Scale1 update
    Scale2.update();                                //Do a Scale2 update
  }
  while (digitalRead(Enterbtn)==0){                 //Check ENTER button status
    delay(50);                                      //Wait for ENTER button release
    Scale1.update();                                //Do a Scale1 update
    Scale2.update();                                //Do a Scale2 update
  }
}

//Subroutine - Display Wheel Base (Axle Separation)
void Display_WheelBase() {
  lcd.clear();                                      //Clear the screen
  lcd.setCursor(2,0);                               //Set cursor to column 3, row 1
  lcd.print("SET WHEEL BASE");                      //Display screen title
  lcd.setCursor(1,1);                               //Set cursor to column 2, row 2
  lcd.print("Separation: ");                        //Display prefix
  lcd.print(wheelBase,3);                           //Display value to 3 decimal places
  lcd.print(char(34));                              //Display " for inches
  lcd.setCursor(0,3);                               //Set cursor to column 1, row 4
  Ocg = 0.1714 * wheelBase;                         //Calculate optimum CG position
  lcd.print("Ideal CG Pos: ");                      //Display optimum CG position
  lcd.print(Ocg,3);                                 //Display value to 3 decimal places
  lcd.print(char(34));                              //Display " for inches
}

//Subroutine - Display Calibration Weight
void Display_Cal_Weight() {
  lcd.clear();                                      //Clear the screen
  lcd.setCursor(1,0);                               //Set cursor to column 2, row 1
  lcd.print("CALIBRATION WEIGHT");                  //Display screen title
  lcd.setCursor(0,2);                               //Set cursor to column 1, row 3
  lcd.print("Cal Wt: ");                            //Print the prefix text
  lcd.print(calMass);                               //Display value
  lcd.print(" oz");                                 //Display suffix
}

//Subroutine - Display Weights & CG
void Display_Weights() {
  lcd.home();
  lcd.clear();                                      //Clear the screen
  lcd.setCursor(6,0);                               //Set cursor to column 7, row 1
  lcd.print("WEIGHTS");                             //Display screen title
  lcd.setCursor(0,1);                               //Set cursor to column 0, row 2
  lcd.print("R:");                                  //Print the prefix text for Scale1 (Rear Axle)
  lcd.print(Scale1_value);                          //Write Scale1 value rounded to 2 decimal places
  lcd.print(" oz");                                 //Display suffix
  lcd.setCursor(11,1);                              //Set cursor to column 11, row 2
  lcd.print("F:");                                  //Print the prefix text for Scale2 (Front Axle)
  lcd.print(Scale2_value);                          //Write Scale1 value rounded to 2 decimal places
  lcd.print(" oz");                                 //Display suffix
  lcd.setCursor(1,2);                               //Set cursor to column 2, row 3
  lcd.print("Total Wt: ");                          //Print the prefix text for total weight
  lcd.print(Scale_Sum);                             //Print the total weight
  lcd.print(" oz");                                 //Display suffix
  lcd.setCursor(4,3);                               //Set cursor to column 5, row 4
  lcd.print("CG: ");                                //Print the prefix text CG
  lcd.print(CG, 3);                                 //Write CG value to 3 decimal places
  lcd.print(" in");                                 //Display suffix
}

//Subroutine - Display Calibration Warning Msg
void Display_Cal_Warning() {
  lcd.clear();                                      //Clear the screen
  lcd.setCursor(6,0);                               //Set cursor to column 7, row 1
  lcd.print("WARNING ");                            //Display screen text
  lcd.setCursor(2,1);                               //Set cursor to column 3, row 2
  lcd.print("Not Calibrated");                      //Display screen text
  lcd.setCursor(0,2);                               //Set cursor to column 1, row 3
  lcd.print("Perfrom Calibration");                 //Display screen text
}

//Subroutine - Display Clear Line
void lcdClearline(int x) {
  if (x>=0 && x<4) {                                //Check valid range of x (0, 1, 2, 3)
    lcd.setCursor(0,x);                             //Set cursor to column 1 of selected row
    lcd.print("                    ");              //Erase any characters of selected row
    lcd.setCursor(0,x);                             //Set cursor back to column 1 of selected row
  }
}

//Scale Calibration Routine
void Calibrate_Scale(int y) {                       //y=1 for left scale, y=2 for right scale
  lcd.clear();                                      //Clear the screen
  lcd.setCursor(1,0);                               //Set cursor to column 2, row 1
  if (y==1) lcd.print("CALIBRATE LT SCALE");        //Display screen text
  if (y==2) lcd.print("CALIBRATE RT SCALE");        //Display screen text
  lcd.setCursor(0,1);                               //Set cursor to column 1, row 2
  lcd.print("Step 1: Remove any");                  //Display screen text
  lcd.setCursor(0,2);                               //Set cursor to column 1, row 3
  lcd.print(" weights from scale");                 //Display screen text
  lcd.setCursor(0,3);                               //Set cursor to column 1, row 4
  lcd.print("Press ENTER to cont.");                //Display screen text
  Waitfor_Enter_Button();                           //Call Enter button press subroutine
  while(1) {                                        //Loop until break command
    if (y==1) {                                     //Do if the left scale
      Scale1.update();                              //Do a Scale1 update
      while (!Scale1.update());                     //Wait for update to finish
      Scale1.tare();                                //Do Scale1 tare measurement
      if (Scale1.getTareStatus()==true) {           //Exit while loop when done
        break;
      }
    }
    if (y==2) {                                     //Do if the right scale
      Scale2.update();                              //Do a Scale2 update
      while (!Scale2.update());                     //Wait for update to finish
      Scale2.tare();                                //Do Scale2 tare measurement
      if (Scale2.getTareStatus()==true) {           //Exit while loop when done
        break;
      }
    }
  }
  lcdClearline(1);                                  //Call subroutine to clear row 2
  lcd.print("Step 2: Place Cal WT");                //Display screen text
  lcdClearline(2);                                  //Call subroutine to clear row 3
  if (y==1) lcd.print("   on left scale");          //Display screen text
  if (y==2) lcd.print("   on right scale");         //Display screen text
  Waitfor_Enter_Button();                           //Call Enter button press subroutine
  lcdClearline(1);                                  //Call subroutine to clear row 2
  lcdClearline(3);                                  //Call subroutine to clear row 4
  lcdClearline(2);                                  //Call subroutine to clear row 3
  lcd.print("Calibrating....");                     //Display screen text
  if (y==1) {                                       //Do if the left scale
    Scale1.update();                                //Do a Scale1 update
    Scale1.refreshDataSet();                        //Refresh dataset to be sure known mass is measured correct
    newValue = Scale1.getNewCalibration(calMass);   //get the new calibration value
  }
  if (y==2) {                                       //Do if the right scale
    Scale2.update();                                //Do a Scale2 update
    Scale2.refreshDataSet();                        //Refresh dataset to be sure known mass is measured correct
    newValue = Scale2.getNewCalibration(calMass);   //get the new calibration value
  }
  lcdClearline(1);                                  //Call subroutine to clear row 2
  lcd.print("New Cal Val:");                        //Display screen text
  lcd.setCursor(13,1);                              //Set cursor to column 14, row 2
  lcd.print(newValue);                              //Display new Scale1 calibration factor
  lcdClearline(2);                                  //Call subroutine to clear row 3
  lcd.setCursor(0,3);                               //Set cursor to column 1, row 4
  lcd.print("+:Save,    -:Discard");                //Display screen text
  while (1) {                                       //Loop until a button is pressed
    if (digitalRead(Upbtn)==0){                     //Is Up button pressed?
      if (y==1) {                                   //Do if the left scale
        calFactor1 = newValue;                      //Update Scale1 calibration factor
        EEPROM.put(eepromAddress1, calFactor1);     //Save in Arduino EEPROM
        cal1Flag = true;                            //Set Scale1 calibrated flag to true
        break;                                      //Exit while routine
      }
      if (y==2) {                                   //Do if the right scale
        calFactor2 = newValue;                      //Update Scale1 calibration factor
        EEPROM.put(eepromAddress2, calFactor2);      //Save in Arduino EEPROM
        cal2Flag = true;                            //Set Scale1 calibrated flag to true
        break;                                      //Exit while routine
      }
    }
    if (digitalRead(Dnbtn)==0){                     //Is Down button pressed?
        break;                                      //Exit while routine
    }
    Scale1.update();                                //Do a Scale1 update
    Scale2.update();                                //Do a Scale2 update
  }
  lcdClearline(1);                                  //Call subroutine to clear row 2
  lcd.print("Step 3: Remove Cal");                  //Display screen text
  lcdClearline(2);                                  //Call subroutine to clear row 3
  lcd.print(" weight from scale");                  //Display screen text
  lcdClearline(3);                                  //Call subroutine to clear row 4
  lcd.print("Press ENTER to cont.");                //Display screen text
  Waitfor_Enter_Button();                           //Call Enter button press subroutine
  lcd.clear();                                      //Clear the screen
  lcd.setCursor(0,0);                               //Set cursor to column 1, row 1
  lcd.print("CALIBRATION COMPLETE");                //Display screen text
  lcd.setCursor(0,1);                               //Set cursor to column 1, row 2
  lcd.print("Place CAL switch OFF");                //Display screen text
  lcd.setCursor(0,2);                               //Set cursor to column 1, row 3
  lcd.print(" or press ENTER to");                  //Display screen text
  lcd.setCursor(0,3);                               //Set cursor to column 1, row 4
  lcd.print(" rerun calibration");                  //Display screen text
  while (digitalRead(Cal_Sw1)==0 || digitalRead(Cal_Sw2)==0) { //Loop while Cal switch is set
    if (digitalRead(Enterbtn)==0){                  //Check Enter button status for press
      break;                                        //And exit while loop if pressed
    }
  }
}
  
